package com.persagy.filemove.dto;

import com.persagy.filemove.app.SagaFileMoveApp;
import com.persagy.filemove.service.SagaFileMoveService;
import com.persagy.filemove.util.JsonTools;
import javafx.application.Platform;
import javafx.concurrent.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class WorkerFactory {
    private static final Logger log = LoggerFactory.getLogger(WorkerFactory.class);

    /**
     * 获取执行参数校验的worker线程实例
     * @param app
     * @return
     */
    public static Task<Boolean> getValidParamWorker(SagaFileMoveApp app) {
        return new Task<Boolean>() {
            @Override
            protected Boolean call() throws Exception {
                updateProgress(-1, 1);
                String result = SagaFileMoveService.getParamValidResult();
                updateProgress(0, 1);

                Platform.runLater(()->{
                    if(result.startsWith("success")) {
                        if(result.contains("warning")) {
                            app.showTipsWarn(result);
                        }else {
                            app.showTipsSuccess(result);
                        }
                    }else {
                        app.showTipsError(result);
                    }
                });

                app.disableSet(false);

                return true;
            }
        };
    }


    public static Task<Boolean> getTransRateWorker(SagaFileMoveApp app) {
        return new Task<Boolean>() {
            @Override
            protected Boolean call() throws Exception {
                boolean paramOK = false;
                updateMessage("准备...");
                updateProgress(-1, 1);
                Thread.sleep(1000);

                // 获取校验参数结果
                String result = SagaFileMoveService.getParamValidResult();

                if(result.startsWith("success")) {
                    // 校验通过

                    updateMessage("获取待迁移资源的key信息...");
                    Thread.sleep(1000);

                    ResourceDTO resourceDTO = SagaFileMoveService.requestSourceKey();

                    updateProgress(0, 1);
                    if(null != resourceDTO.errorMsg) {
                        // 获取资源key信息出错
                        updateMessage(resourceDTO.errorMsg);
                    }else {
                        Map<String, String> obj_key = resourceDTO.obj_key;

                        if(obj_key.isEmpty()) {
                            // 没有要转移的数据
                            updateMessage("结束。查询到"+SagaFileMoveApp.dto.objType+"对象记录数为0。");
                        }else {
                            updateMessage("开始迁移数据，资源文件总数量：" + obj_key.size());
                            updateTitle("0.0%");

                            // 文件不存在信息集合
                            Set<TransErrDTO> noFileSet = new HashSet<>();
                            // 资源key为null信息集合
                            Set<TransErrDTO> nullKeySet = new HashSet<>();
                            // 文件转移失败信息集合
                            Set<TransErrDTO> moveErrorSet = new HashSet<>();

                            // 要转移资源的总数量
                            int totalNum = obj_key.size();

                            int movedNum = 0;
                            for(String objId : obj_key.keySet()) {
                                SagaFileMoveService.fileMove(objId, obj_key.get(objId), noFileSet, nullKeySet, moveErrorSet);
                                System.out.println("--- " + movedNum);
                                updateProgress(++movedNum, totalNum);
                                updateTitle(String.format("%.1f", 100.0 * movedNum / totalNum)+"%");
                            }

                            int noFileNum = noFileSet.size(), nullKeyNum = nullKeySet.size(), moveErrNum = moveErrorSet.size();
                            int errorNum = noFileNum + nullKeyNum + moveErrNum;

                            if(errorNum > 0) {

                                StringBuilder sb = null;

                                if(noFileNum > 0) {
                                    sb = new StringBuilder("对象关联的文件不存在的数量：" + noFileNum + "。详情：\n");
                                    for(TransErrDTO item : noFileSet) {
                                        sb.append(JsonTools.beanToJson(item) + "\n");
                                    }
                                    log.error(sb.toString());
                                }

                                if(nullKeyNum > 0) {
                                    sb = new StringBuilder("对象关联文件key为空的数量：" + nullKeyNum + "。详情：\n");
                                    for(TransErrDTO item : nullKeySet) {
                                        sb.append(JsonTools.beanToJson(item) + "\n");
                                    }
                                    log.error(sb.toString());
                                }

                                if(moveErrNum > 0) {
                                    sb = new StringBuilder("文件迁移出错的数量：" + moveErrNum + "。详情：\n");
                                    for(TransErrDTO item : moveErrorSet) {
                                        sb.append(JsonTools.beanToJson(item) + "\n");
                                    }
                                    log.error(sb.toString());
                                }
                            }

                            if(errorNum > 0) {
                                updateMessage("迁移完成。共：" + totalNum + ", 出错：" + errorNum + "。详情请见日志信息。");
                            }else {
                                updateMessage("迁移完成。");
                            }
                        }
                    }
                }else {
                    // 校验失败
                    updateProgress(0, 1);
                    updateMessage(result);
                }

                app.disableSet(false);

                return true;
            }
        };
    }

}
