package com.persagy.filemove.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.filemove.app.SagaFileMoveApp;
import com.persagy.filemove.dto.ResourceDTO;
import com.persagy.filemove.dto.TransErrDTO;
import com.persagy.filemove.util.HttpTools;
import com.persagy.filemove.util.JsonTools;
import javafx.beans.property.SimpleStringProperty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class SagaFileMoveService {
    private static final Logger log = LoggerFactory.getLogger(SagaFileMoveService.class);
    public static final String resp_success = "success";
    public static final String testKey = "persagyAndSagaTest2020121130";
    public static final String url_pj_get = "/mng/project/query";
    public static final String url_obj_query = "/object/subset_query?projectId=PJID&secret=SEC";
    public static final String url_source_get = "/common/TYPE_get?systemId=SYSID&key=";
    public static final String url_source_send = "/common/TYPE_upload?systemId=SYSID&secret=SEC&overwrite=true&key=";

    public static SimpleStringProperty rateProperty = new SimpleStringProperty("");


    /**
     * 检查数据平台地址和项目信息
     * @return
     */
    public static boolean validParamDpfAndPj() {
        boolean result = false;
        Map<Object, Object> param = new HashMap<>();
        param.put("projectId", SagaFileMoveApp.dto.pjId.getValue());

        String reqUrl = SagaFileMoveApp.dto.dpf.getValue() + url_pj_get;
        String resp = null;
        try {
            resp = HttpTools.httpPostJson(reqUrl, param);
            if(null != resp && resp.contains(resp_success) && resp.contains(SagaFileMoveApp.dto.pjId.getValue())) {
                result = true;
            }
        }catch (Exception e) {
            log.error("url:" + reqUrl + ", param:" + param + ", result:" + resp + "errMsg:" + e.getMessage());
        }
        return result;
    }

    /**
     * 校验参数
     * @return 参数无误，返回"success-信息"；
     */
    public static String getParamValidResult() {
        String errorMsg = null;

        if(!SagaFileMoveApp.dto.validNecessaryParam()) {
            errorMsg = "必填参数不能为空！";
        }

        if(errorMsg == null && !validParamDpfAndPj()) {
            errorMsg = "获取不到项目信息！请确保数据平台地址和项目id参数正确。";
        }

        if(errorMsg == null && !validParamImgFromUrl()) {
            errorMsg = "[文件服务-From]访问不通！请确保URL正确。";
        }

        if(errorMsg == null && !validParamImgToUrl()) {
            errorMsg = "[文件服务-To]访问不通！请确保URL正确。";
        }

        if(errorMsg == null) {
            if(SagaFileMoveApp.dto.imgFromUrl.getValue().equals(SagaFileMoveApp.dto.imgToUrl.getValue())) {
                errorMsg = "success-校验通过。warning-两个文件服务的URL相同！";
            }else {
                errorMsg = "success-校验通过";
            }
        }

        return errorMsg;
    }

    /**
     * 在两个文件服务器之间转移文件
     * @param objId 对象id
     * @param key 文件资源的key
     * @param noFileSet 文件不存在错误信息集合
     * @param nullKeySet 资源key为null的信息集合
     * @param moveErrorSet 文件转移失败的信息集合
     */
    public static void fileMove(String objId, String key, Set<TransErrDTO> noFileSet, Set<TransErrDTO> nullKeySet, Set<TransErrDTO> moveErrorSet) {
        if(key == null || key.trim().length() == 0) {
            // 文件资源key值为空
            nullKeySet.add(new TransErrDTO(objId, key, "", "", ""));
            return;
        }

        String getImgUrl = buildGetResourceUrl(key);
        String sendUrl = buildUploadResourceUrl(key);

        byte[] resourceByteData = null;

        try {
            Thread.sleep(3000);
            resourceByteData = HttpTools.httpGetFile(getImgUrl);
        }catch (Exception e) {
            e.printStackTrace();
            moveErrorSet.add(new TransErrDTO(objId, key, "getResourceError:" + e.getMessage(), getImgUrl, sendUrl));
        }

        if(resourceByteData != null) {
            if(resourceByteData.length < 50 && ifNotExists(resourceByteData)) {
                // 文件不存在
                noFileSet.add(new TransErrDTO(objId, key, "resourceNotExist", getImgUrl, ""));
            }else {
                String uploadResult = null;

                try {
                    uploadResult = HttpTools.httpPostRequest(sendUrl, resourceByteData);
                }catch (Exception e){
                    e.printStackTrace();
                    // 文件转移报错
                    moveErrorSet.add(new TransErrDTO(objId, key, "uploadResourceError:" + e.getMessage(), getImgUrl, sendUrl));
                }

                if(null == uploadResult || !uploadResult.contains(resp_success)) {
                    // 文件转移失败
                    moveErrorSet.add(new TransErrDTO(objId, key, "uploadResourceFailed：" + uploadResult, getImgUrl, sendUrl));
                }
            }
        }
    }

    /**
     * 获取上传资源Url
     * @param key 资源key
     * @return
     */
    private static String buildUploadResourceUrl(String key) {
        // 默认使用file_upload上传文件
        String toImgApi = "file";
        if("image".equals(SagaFileMoveApp.dto.imgToApiType.getValue())) {
            toImgApi = "image";
        }

        String sendUrl = SagaFileMoveApp.dto.imgToUrl.getValue() + url_source_send;
        sendUrl = sendUrl.replace("TYPE", toImgApi)
                .replace("SYSID", SagaFileMoveApp.dto.imgToSysId.getValue())
                .replace("SEC", SagaFileMoveApp.dto.imgToSecret.getValue())
                + key;

        return sendUrl;
    }

    /**
     * 构建获取资源的Url
     * @param key 文件资源的key
     * @return
     */
    private static String buildGetResourceUrl(String key) {
        // 下载资源使用的接口，默认使用image_get接口
        String getImgApi = "image";

        if("file".equals(SagaFileMoveApp.dto.imgFromApiType.getValue())) {
            // UI指定使用file_get下载文件
            getImgApi = "file";
        }

        // 获取资源的url
        String getImgUrl = SagaFileMoveApp.dto.imgFromUrl.getValue() + url_source_get;
        getImgUrl = getImgUrl.replace("TYPE", getImgApi)
                .replace("SYSID", SagaFileMoveApp.dto.imgFromSysId.getValue());

        getImgUrl = getImgUrl + key;

        if(key.endsWith(".jsonz")) {
            // 对于.jsonz文件，从file_get接口获取
            getImgUrl = getImgUrl.replace("image_get","file_get");
        }

        return getImgUrl;
    }

    /**
     * 从数据平台获取对象关联的资源key
     * @return
     */
    public static ResourceDTO requestSourceKey() {
        ResourceDTO result = new ResourceDTO();
        Map<String, String> obj_key = new HashMap<>();
        result.obj_key = obj_key;

        log.info("查询数据平台对象，获取文件资源key...");

        String reqUrl = SagaFileMoveApp.dto.dpf.getValue() + url_obj_query;
        reqUrl = reqUrl.replace("PJID", SagaFileMoveApp.dto.pjId.getValue()).replace("SEC", SagaFileMoveApp.dto.pjSecret.getValue());

        JSONObject paramObj = new JSONObject();

        JSONObject criteria = new JSONObject();
        JSONArray typeArray = new JSONArray();
        typeArray.add(SagaFileMoveApp.dto.objType.getValue());
        criteria.put("type", typeArray);

        JSONArray returnInfos = new JSONArray();
        returnInfos.add(SagaFileMoveApp.dto.objInfoCode.getValue());

        paramObj.put("customInfo", true);
        paramObj.put("criteria", criteria);
        paramObj.put("returnInfos", returnInfos);

        String resp = null;
        try{
            resp = HttpTools.httpPostJson(reqUrl, paramObj);
        }catch (Exception e){
            e.printStackTrace();
            log.error("请求数据平台出错！url:" + reqUrl + ", param:" + paramObj + ", result:" + resp + "errMsg:" + e.getMessage());
            result.errorMsg = "请求数据平台出错！";
            return result;
        }

        if(null != resp) {
            JSONObject respJson = null;
            try {
                respJson = JsonTools.strToJsonObj(resp);
            }catch (Exception e) {
                e.printStackTrace();
                log.error("String转Json对象失败：" + e.getMessage());
                result.errorMsg = "String转Json对象失败：" + e.getMessage();
                return result;
            }

            if(respJson != null && resp_success.equals(respJson.getString("Result"))) {
                JSONArray content = respJson.getJSONArray("Content");
                if(null != content && !content.isEmpty()) {
                    String objId = null, sourceKey = null;

                    for(int i=0; i<content.size(); i++) {
                        objId = content.getJSONObject(i).getString("id");
                        JSONObject infoJson = content.getJSONObject(i).getJSONObject("infos");

                        if(null != infoJson) {
                            sourceKey = infoJson.getString(SagaFileMoveApp.dto.objInfoCode.getValue());
                        }
                        obj_key.put(objId, sourceKey);
                    }

                }else {
                    log.info("结束。查询到数据平台对象数量为0。");
                }
            }else {
                result.errorMsg = "请求数据平台出错：" + resp;
                log.error("请求数据平台出错：" + resp);
            }
        }

        return result;
    }

    /**
     * 校验文件服务From是否可访问
     * @return
     */
    public static boolean validParamImgFromUrl() {
        return fileGetTest(SagaFileMoveApp.dto.imgFromUrl.getValue(),  SagaFileMoveApp.dto.imgFromSysId.getValue());
    }

    /**
     * 校验文件服务From是否可访问
     * @return
     */
    public static boolean validParamImgToUrl() {
        return fileGetTest(SagaFileMoveApp.dto.imgToUrl.getValue(), SagaFileMoveApp.dto.imgToSysId.getValue());
    }

    private static boolean fileGetTest(String base, String sysId) {
        boolean result = false;
        String url = base + url_source_get.replace("TYPE", "file").replace("SYSID", sysId) + testKey;
        try {
            byte[] byteData = HttpTools.httpGetFile(url);
            if(byteData != null && ifNotExists(byteData)) {
                result = true;
            }
        }catch (Exception e){}

        return result;
    }

    /**
     * 判断byte数组是否包含“not existed”字符串
     * @param byteData
     * @return true-包含；false-不包含
     */
    private static boolean ifNotExists(byte[] byteData) {
        String s = new String(byteData);
        System.out.println(s);
        return s.contains("not existed");
    }
}
